--------------------------------------------------------
-- VHDL code per blocco Address Resolution del modulo
-- MEMORY.
-- By Sebastiano Otupacca - PAP1 2008/2009, 06-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity Address_Resolution is

  port(ck       : in std_logic;                          -- Clock
       CLR_N    : in std_logic;                          -- Clear (active-low)
       incpc_N  : in std_logic;                          -- Increment PC (active-low)
       ldpc_N   : in std_logic;                          -- Load PC (active-low)
       rdpc_N   : in std_logic;                          -- Read PC (active-low)
       incmar_N : in std_logic;                          -- Increment MAR - Memory Address Register (active-low)
       ldmar_N  : in std_logic;                          -- Load MAR - Memory Address Register (active-low)
       spc      : in std_logic;                          -- Select PC
       Addr_out : out std_logic_vector(15 downto 0);     -- Bus indirizzi (16 bit)
       DATA     : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU

end Address_Resolution;

--------------------------------------------------------

architecture struct of Address_Resolution is

  component NOT1
    port(D : in std_logic;
         Q : out std_logic);
  end component;

  component MUX2
    generic(n: natural := 8);
    port(sel : in std_logic;
         D0  : in std_logic_vector(n-1 downto 0);
         D1  : in std_logic_vector(n-1 downto 0);
         Q   : out std_logic_vector(n-1 downto 0));
  end component;

  component Reg
    generic(n: natural := 8);
    port(clk   : in std_logic;
         CLR_N : in std_logic;
         EN_N  : in std_logic;
         D     : in std_logic_vector(n-1 downto 0);
         Q     : out std_logic_vector(n-1 downto 0));
  end component;

  component BufZ
    generic(n: natural := 8);
    port(OE : in std_logic;
         D  : in std_logic_vector(n-1 downto 0);
         Q  : out std_logic_vector(n-1 downto 0));
  end component;

  component Adder1
    generic(n: natural := 8);
    port(D : in std_logic_vector(n-1 downto 0);
         Q : out std_logic_vector(n-1 downto 0));
  end component;

  component AND2
    port(A : in std_logic;
         B : in std_logic;
         Q : out std_logic);
  end component;

  -- Input, output e abilitazione del PC
  signal PC_source   : std_logic_vector(15 downto 0);
  signal PC_out      : std_logic_vector(15 downto 0);
  signal PC_enable_N : std_logic;

  -- Input, output e abilitazione del MAR
  signal MAR_source   : std_logic_vector(15 downto 0);
  signal MAR_out      : std_logic_vector(15 downto 0);
  signal MAR_enable_N : std_logic;

  -- Input dell'Adder1
  signal Adder_source : std_logic_vector(15 downto 0);
  signal Adder_out    : std_logic_vector(15 downto 0);

  signal ldpc   : std_logic;
  signal incmar : std_logic;
  signal spc_N  : std_logic;
  signal rdpc   : std_logic;

begin

  -- Istanzia la porta NOT di negazione del segnale ldpc_N
  blk_NOT_1: NOT1 port map(D => ldpc_N,
                           Q => ldpc);

  -- Istanzia il MUX di selezione dell'input del PC (Program Counter)
  blk_MUX_PC: MUX2 generic map(n => 16)
                   port map(sel => ldpc,        -- Selector tra PC_inc e PC_load
                            D0  => Adder_out,   -- Input #0 = Output dell'Adder1 (incremento)
                            D1  => DATA,        -- Input #1 = Bus dati della CPU
                            Q   => PC_source);  -- Output verso il PC

  -- Istanzia il MUX di selezione dell'input del MAR (Memory Address Register)
  blk_MUX_MAR: MUX2 generic map(n => 16)
                    port map(sel => incmar_N,     -- Selector tra MAR_inc e MAR_load
                             D0  => Adder_out,    -- Input #0 = Output dell'Adder1 (incremento)
                             D1  => DATA,         -- Input #1 = Bus dati della CPU
                             Q   => MAR_source);  -- Output verso il MAR

  -- Istanzia la porta NOT di negazione del segnale incmar_N
  blk_NOT_2: NOT1 port map(D => incmar_N,
                           Q => incmar);

  -- Istanzia il MUX di selezione dell'input dell'Adder1
  blk_MUX_Add: MUX2 generic map(n => 16)
                    port map(sel => incmar,         -- Selector tra PC_out e MAR_out
                             D0  => PC_out,         -- Input #0 = Output del PC
                             D1  => MAR_out,        -- Input #1 = Output del MAR
                             Q   => Adder_source);  -- Output verso l'Adder1

  -- Istanzia la porta NOT di negazione del segnale spc
  blk_NOT_3: NOT1 port map(D => spc,
                           Q => spc_N);

  -- Istanzia il MUX di selezione dell'uscita del bus indirizzi
  blk_MUX_Adr: MUX2 generic map(n => 16)
                    port map(sel => spc_N,      -- Selector tra PC_out e MAR_out
                             D0  => PC_out,     -- Input #0 = Output del PC
                             D1  => MAR_out,    -- Input #1 = Output del MAR
                             Q   => Addr_out);  -- Output verso il bus indirizzi

  -- Istanzia l'Adder1 (incremento)
  blk_Adder1: Adder1 generic map(n => 16)
                     port map(D => Adder_source,
                              Q => Adder_out);

  -- Istanzia la porta AND a 2 input
  blk_ANDpc: AND2 port map(A => incpc_N,
                           B => ldpc_N,
                           Q => PC_enable_N);

  -- Istanzia il registro PC (Program Counter)
  blk_PC: Reg generic map(n => 16)
              port map(clk   => ck,           -- Clock
                       CLR_N => CLR_N,        -- Clear (globale)
                       EN_N  => PC_enable_N,  -- Abilitazione del registro
                       D     => PC_source,    -- Dall'uscita del MUX Mem_out/Byte_Select
                       Q     => PC_out);      -- All'input dati del blocco Main_Memory

  -- Istanzia la porta AND a 2 input
  blk_ANDmar: AND2 port map(A => incmar_N,
                            B => ldmar_N,
                            Q => MAR_enable_N);

  -- Istanzia il registro MAR (Memory Address Register)
  blk_MAR: Reg generic map(n => 16)
               port map(clk   => ck,            -- Clock
                        CLR_N => CLR_N,         -- Clear (globale)
                        EN_N  => MAR_enable_N,  -- Abilitazione del registro
                        D     => MAR_source,    -- Dall'uscita del MUX Mem_out/Byte_Select
                        Q     => MAR_out);      -- All'input dati del blocco Main_Memory

  -- Istanzia la porta NOT di negazione del segnale rdpc_N
  blk_NOT_4: NOT1 port map(D => rdpc_N,
                           Q => rdpc);

  -- Istanzia un buffer a tre stati per la lettura del PC (16 bit di dati)
  blk_Buf: BufZ generic map(n => 16)
                port map(OE => rdpc,    -- Output enable, attivo quando rdpc_N  attivo
                         D  => PC_out,  -- Input dall'uscita del PC
                         Q  => DATA);   -- Output sul Bus dati della CPU

end struct;

--------------------------------------------------------
