--------------------------------------------------------
-- VHDL code per blocco MDR Control del modulo MEMORY.
-- By Sebastiano Otupacca - PAP1 2008/2009, 06-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity MDR_Control is

  port(ck       : in std_logic;                          -- Clock
       CLR_N    : in std_logic;                          -- Clear (active-low)
       rd_N     : in std_logic;                          -- Read enable (active-low)
       rdmdr_N  : in std_logic;                          -- Read Memory Data
       ldmdr_N  : in std_logic;                          -- Load Memory Data (active-low)
       ldhmdr_N : in std_logic;                          -- Load Memory Data - High byte (active-low)
       Mem_data : in std_logic_vector(7 downto 0);       -- Dall'output dati del blocco Main_Memory
       MDR_out  : out std_logic_vector(7 downto 0);      -- All'input dati del blocco Main_Memory
       DATA     : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU

end MDR_Control;

--------------------------------------------------------

architecture struct of MDR_Control is

  component NOT1
    port(D : in std_logic;
         Q : out std_logic);
  end component;

  component MUX2
    generic(n: natural := 8);
    port(sel : in std_logic;
         D0  : in std_logic_vector(n-1 downto 0);
         D1  : in std_logic_vector(n-1 downto 0);
         Q   : out std_logic_vector(n-1 downto 0));
  end component;

  component Reg
    generic(n: natural := 8);
    port(clk   : in std_logic;
         CLR_N : in std_logic;
         EN_N  : in std_logic;
         D     : in std_logic_vector(n-1 downto 0);
         Q     : out std_logic_vector(n-1 downto 0));
  end component;

  component BufZ
    generic(n: natural := 8);
    port(OE : in std_logic;
         D  : in std_logic_vector(n-1 downto 0);
         Q  : out std_logic_vector(n-1 downto 0));
  end component;

  component Merge_Vectors
    generic(nV0: natural := 8; nV1: natural := 8);
    port(v0 : in std_logic_vector(nV0-1 downto 0);        -- Vector low
         v1 : in std_logic_vector(nV1-1 downto 0);        -- Vector high
         Q  : out std_logic_vector(nV0+nV1-1 downto 0));  -- Vector merge output
  end component;

  component AND2
    port(A : in std_logic;
         B : in std_logic;
         Q : out std_logic);
  end component;

  -- Output del Merge Vectors
  signal Data16 : std_logic_vector(15 downto 0);

  -- Output della porta AND a 2 input
  signal MDR_enable_N : std_logic;

  -- Output del MUX di selezione Low-byte/High-byte
  signal Byte_Select : std_logic_vector(7 downto 0);

  -- Output del MUX => input registro MDR
  signal MDR_source   : std_logic_vector(7 downto 0);
  signal MDR_internal : std_logic_vector(7 downto 0);

  signal rdmdr  : std_logic;
  signal ldhmdr : std_logic;

begin

  -- Istanzia la porta NOT di negazione del segnale rdmdr_N
  blk_NOT_1: NOT1 port map(D => rdmdr_N,
                           Q => rdmdr);

  -- Istanzia un buffer a tre stati (16 bit di dati)
  blk_Buf: BufZ generic map(n => 16)
                port map(OE => rdmdr,   -- Output enable, attivo quando rdmdr_N  attivo
                         D  => Data16,  -- Input dall'uscita del Merge_Vectors
                         Q  => DATA);   -- Output sul Bus dati della CPU

  -- Istanzia la porta NOT di negazione del segnale ldhmdr_N
  blk_NOT_2: NOT1 port map(D => ldhmdr_N,
                           Q => ldhmdr);

  -- Istanzia il MUX di selezione Low-byte/High-byte
  blk_MUX_H_L: MUX2 port map(sel => ldhmdr,             -- Selector tra Low-byte/High-byte
                             D0  => DATA(7 downto 0),   -- Input #0 = Low-byte del Bus dati della CPU
                             D1  => DATA(15 downto 8),  -- Input #1 = High-byte del Bus dati della CPU
                             Q   => Byte_Select);       -- Output del MUX2

  -- Istanzia il MUX di selezione Mem_out/Byte_Select
  blk_MUX2: MUX2 port map(sel => rd_N,           -- Selector tra Mem_out e Low/High Data bus
                          D0  => Mem_data,       -- Input #0 = Output dati del blocco Main_Memory (TTY mappato in memoria)
                          D1  => Byte_Select,    -- Input #1 = Low-byte/High-byte del Bus dati della CPU
                          Q   => MDR_source);  -- Output del MUX2

  -- Istanzia la porta AND a 2 input
  blk_AND2: AND2 port map(A => ldmdr_N,
                          B => ldhmdr_N,
                          Q => MDR_enable_N);

  -- Istanzia il registro MDR
  blk_MDR: Reg port map(clk   => ck,             -- Clock
                        CLR_N => CLR_N,          -- Clear (globale)
                        EN_N  => MDR_enable_N,   -- Abilitazione del registro
                        D     => MDR_source,     -- Dall'uscita del MUX Mem_out/Byte_Select
                        Q     => MDR_internal);  -- All'input dati del blocco Main_Memory

  -- Istanzia il Merge Vectors, Low-byte concatenato con High-byte in una word a 16 bit
  blk_MergeV: Merge_Vectors port map(v0 => MDR_internal,
                                     v1 => (others => '0'),
                                     Q  => Data16);

  -- Assegna il segnale MDR_internal all'output
  MDR_out <= MDR_internal;

end struct;

--------------------------------------------------------
