--------------------------------------------------------
-- VHDL code per modulo MEMORY.
-- By Sebastiano Otupacca - PAP1 2008/2009, 06-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity MEMORY is

  port(ck                : in std_logic;                          -- Clock
       CLR_N             : in std_logic;                          -- Clear (active-low)
       rd_N              : in std_logic;                          -- Read enable (active-low)
       wrt_N             : in std_logic;                          -- Write enable (active-low)
       incpc_N           : in std_logic;                          -- Increment PC (active-low)
       ldpc_N            : in std_logic;                          -- Load PC (active-low)
       rdpc_N            : in std_logic;                          -- Read PC (active-low)
       incmar_N          : in std_logic;                          -- Increment MAR - Memory Address Register (active-low)
       ldmar_N           : in std_logic;                          -- Load MAR - Memory Address Register (active-low)
       ldmdr_N           : in std_logic;                          -- Load Memory Data (active-low)
       ldhmdr_N          : in std_logic;                          -- Load Memory Data - High byte (active-low)
       rdmdr_N           : in std_logic;                          -- Read Memory Data
       spc               : in std_logic;                          -- Select PC
       TTY_statusAddrMap : in std_logic_vector(15 downto 0);      -- Indirizzo di mappatura del TTY_Status nella memoria
       TTY_dataAddrMap   : in std_logic_vector(15 downto 0);      -- Indirizzo di mappatura del TTY_Data nella memoria
       RTS_in            : in std_logic;                          -- Request To Send dal TTY
       DTR_in            : in std_logic;                          -- Data Terminal Ready dal TTY
       TTY_data_recv     : in std_logic_vector(7 downto 0);       -- Dati ricevuti dal TTY
       CTS_out           : out std_logic;                         -- Clear To Send verso il TTY
       DSR_out           : out std_logic;                         -- Data Set Ready verso il TTY
       TTY_data_send     : out std_logic_vector(7 downto 0);      -- Dati inviati tramite il TTY
       DATA              : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU

end MEMORY;

--------------------------------------------------------

architecture struct of MEMORY is

  component Address_Resolution
    port(ck       : in std_logic;                          -- Clock
         CLR_N    : in std_logic;                          -- Clear (active-low)
         incpc_N  : in std_logic;                          -- Increment PC (active-low)
         ldpc_N   : in std_logic;                          -- Load PC (active-low)
         rdpc_N   : in std_logic;                          -- Read PC (active-low)
         incmar_N : in std_logic;                          -- Increment MAR - Memory Address Register (active-low)
         ldmar_N  : in std_logic;                          -- Load MAR - Memory Address Register (active-low)
         spc      : in std_logic;                          -- Select PC
         Addr_out : out std_logic_vector(15 downto 0);     -- Bus indirizzi (16 bit)
         DATA     : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU
  end component;

  component Main_Memory
    port(rd_N        : in std_logic;                       -- Read enable (active-low)
         wrt_N       : in std_logic;                       -- Write enable (active-low)
         isTTYStatus : in std_logic;                       -- '1' se l'indirizzo  mappato per la lettura/scrittura dello stato TTY
         isTTYData   : in std_logic;                       -- '1' se l'indirizzo  mappato per la lettura/scrittura del Data TTY
         TTY_status  : in std_logic_vector(7 downto 0);    -- Stato del TTY: DTR = bit0, RTS = bit1
         TTY_data    : in std_logic_vector(7 downto 0);    -- Data received da TTY
         Addr_in     : in std_logic_vector(15 downto 0);   -- Bus indirizzi (16 bit)
         Mem_in      : in std_logic_vector(7 downto 0);    -- Input dati del blocco Main_Memory
         Mem_out     : out std_logic_vector(7 downto 0));  -- Output dati del blocco Main_Memory (TTY mappato in memoria)
  end component;

  component MDR_Control
    port(ck       : in std_logic;                          -- Clock
         CLR_N    : in std_logic;                          -- Clear (active-low)
         rd_N     : in std_logic;                          -- Read enable (active-low)
         rdmdr_N  : in std_logic;                          -- Read Memory Data
         ldmdr_N  : in std_logic;                          -- Load Memory Data (active-low)
         ldhmdr_N : in std_logic;                          -- Load Memory Data - High byte (active-low)
         Mem_data : in std_logic_vector(7 downto 0);       -- Dall'output dati del blocco Main_Memory
         MDR_out  : out std_logic_vector(7 downto 0);      -- All'input dati del blocco Main_Memory
         DATA     : inout std_logic_vector(15 downto 0));  -- Bus dati della CPU
  end component;

  component TTY_Control
    port(ck             : in std_logic;                      -- Clock
         CLR_N          : in std_logic;                      -- Clear (active-low)
         wrt_N          : in std_logic;                      -- Write enable (active-low)
         Data_in        : in std_logic_vector(7 downto 0);   -- Data input (8 bit)
         Addr_in        : in std_logic_vector(15 downto 0);  -- Bus indirizzi (16 bit)
         DTR_in         : in std_logic;                      -- Data Terminal Ready dal TTY
         RTS_in         : in std_logic;                      -- Request To Send dal TTY
         TTY_data_recv  : in std_logic_vector(7 downto 0);   -- Dati ricevuti dal TTY
         TTY_statusAddr : in std_logic_vector(15 downto 0);  -- Indirizzo di mappatura del TTY_Status nella memoria
         TTY_dataAddr   : in std_logic_vector(15 downto 0);  -- Indirizzo di mappatura del TTY_Data nella memoria
         DSR_out        : out std_logic;                     -- Data Set Ready verso il TTY
         CTS_out        : out std_logic;                     -- Clear To Send verso il TTY
         TTY_status_out : out std_logic_vector(7 downto 0);  -- Status del TTY
         TTY_data_send  : out std_logic_vector(7 downto 0);  -- Dati inviati tramite il TTY
         TTY_data_thru  : out std_logic_vector(7 downto 0);  -- Dati ricevuti dal TTY e passati al blocco Main_Memory
         isTTYStatus    : out std_logic;                     -- '1' se l'indirizzo  mappato per la lettura/scrittura dello stato TTY
         isTTYData      : out std_logic);                    -- '1' se l'indirizzo  mappato per la lettura/scrittura del Data TTY
  end component;

  -- Segnali interni
  signal Addr_bus    : std_logic_vector(15 downto 0);
  signal MDR_out     : std_logic_vector(7 downto 0);
  signal MDR_in      : std_logic_vector(7 downto 0);
  signal TTY_status  : std_logic_vector(7 downto 0);
  signal TTY_data    : std_logic_vector(7 downto 0);
  signal isTTYStatus : std_logic;
  signal isTTYData   : std_logic;

begin

  -- Istanzia il blocco Address_Resolution
  blk_1: Address_Resolution port map(ck       => ck,
                                     CLR_N    => CLR_N,
                                     incpc_N  => incpc_N,
                                     ldpc_N   => ldpc_N,
                                     rdpc_N   => rdpc_N,
                                     incmar_N => incmar_N,
                                     ldmar_N  => ldmar_N,
                                     spc      => spc,
                                     Addr_out => Addr_bus,  -- verso i blocchi Main_Memory e TTY_Control
                                     DATA     => DATA);

  -- Istanzia il blocco Main_Memory
  blk_2: Main_Memory port map(rd_N        => rd_N,
                              wrt_N       => wrt_N,
                              isTTYStatus => isTTYStatus,  -- dal blocco TTY_Control
                              isTTYData   => isTTYData,    -- dal blocco TTY_Control
                              TTY_status  => TTY_status,   -- dal blocco TTY_Control
                              TTY_data    => TTY_data,     -- dal blocco TTY_Control
                              Addr_in     => Addr_bus,     -- dal blocco Address_Resolution
                              Mem_in      => MDR_out,      -- dal blocco MDR_Control
                              Mem_out     => MDR_in);      -- verso il blocco MDR_Control

  -- Istanzia il blocco MDR_Control
  blk_3: MDR_Control port map(ck       => ck,
                              CLR_N    => CLR_N,
                              rd_N     => rd_N,
                              rdmdr_N  => rdmdr_N,
                              ldmdr_N  => ldmdr_N,
                              ldhmdr_N => ldhmdr_N,
                              Mem_data => MDR_in,    -- dal blocco Main_Memory
                              MDR_out  => MDR_out,   -- verso il blocco Main_Memory e TTY_Control
                              DATA     => DATA);

-- Istanzia il blocco TTY_Control
  blk_4: TTY_Control port map(ck             => ck,
                              CLR_N          => CLR_N,
                              wrt_N          => wrt_N,
                              Data_in        => MDR_out,            -- dal blocco MDR_Control
                              Addr_in        => Addr_bus,           -- dal blocco Address_Resolution
                              DTR_in         => DTR_in,
                              RTS_in         => RTS_in,
                              TTY_data_recv  => TTY_data_recv,
                              TTY_statusAddr => TTY_statusAddrMap,
                              TTY_dataAddr   => TTY_dataAddrMap,
                              DSR_out        => DSR_out,
                              CTS_out        => CTS_out,
                              TTY_status_out => TTY_status,         -- verso il blocco Main_Memory
                              TTY_data_send  => TTY_data_send,
                              TTY_data_thru  => TTY_data,           -- verso il blocco Main_Memory
                              isTTYStatus    => isTTYStatus,        -- verso il blocco Main_Memory
                              isTTYData      => isTTYData);         -- verso il blocco Main_Memory

end struct;

--------------------------------------------------------
