--------------------------------------------------------
-- VHDL code per blocco MainMemory del modulo MEMORY.
-- By Sebastiano Otupacca - PAP1 2008/2009, 06-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity Main_Memory is

  port(rd_N        : in std_logic;                       -- Read enable (active-low)
       wrt_N       : in std_logic;                       -- Write enable (active-low)
       isTTYStatus : in std_logic;                       -- '1' se l'indirizzo  mappato per la lettura/scrittura dello stato TTY
       isTTYData   : in std_logic;                       -- '1' se l'indirizzo  mappato per la lettura/scrittura del Data TTY
       TTY_status  : in std_logic_vector(7 downto 0);    -- Stato del TTY: DTR = bit0, RTS = bit1
       TTY_data    : in std_logic_vector(7 downto 0);    -- Data received da TTY
       Addr_in     : in std_logic_vector(15 downto 0);   -- Bus indirizzi (16 bit)
       Mem_in      : in std_logic_vector(7 downto 0);    -- Input dati del blocco Main_Memory
       Mem_out     : out std_logic_vector(7 downto 0));  -- Output dati del blocco Main_Memory (TTY mappato in memoria)

end Main_Memory;

--------------------------------------------------------

architecture struct of Main_Memory is

  component RAM
    generic(nData: natural := 8; nAddress: natural := 16);
    port(A    : in std_logic_vector(nAddress-1 downto 0);
         CS_N : in std_logic;
         WE_N : in std_logic;
         OE_N : in std_logic;
         D    : inout std_logic_vector(nData-1 downto 0));
  end component;

  component MUX4
    generic(n: natural := 8);
    port(sel : in std_logic_vector(1 downto 0);
         D0  : in std_logic_vector(n-1 downto 0);
         D1  : in std_logic_vector(n-1 downto 0);
         D2  : in std_logic_vector(n-1 downto 0);
         D3  : in std_logic_vector(n-1 downto 0);
         Q   : out std_logic_vector(n-1 downto 0));
  end component;

  component BufZ
    generic(n: natural := 8);
    port(OE : in std_logic;
         D  : in std_logic_vector(n-1 downto 0);
         Q  : out std_logic_vector(n-1 downto 0));
  end component;

  component OR3
    port(A : in std_logic;
         B : in std_logic;
         C : in std_logic;
         Q : out std_logic);
  end component;

  component Merge_Bits
    port(b0 : in std_logic;
         b1 : in std_logic;
         Q : out std_logic_vector(1 downto 0));
  end component;

  -- Bus di dati interno, collegato alla RAM, al buffer three-state e all'ingesso #0 del MUX
  signal data_bus : std_logic_vector(7 downto 0);

  -- Output della porta OR a 3 input
  signal wrt_enable_N : std_logic;

  -- Output del Merge_Bits
  signal TTY_select : std_logic_vector(1 downto 0);

begin

  -- Istanzia la porta OR a 3 input
  blk_OR3: OR3 port map(A => wrt_N,
                        B => isTTYStatus,
                        C => isTTYData,
                        Q => wrt_enable_N);

  -- Istanzia la RAM con 8 bit di dati e 16 bit di indirizzi
  blk_RAM: RAM port map(A => Addr_in,          -- Bus indirizzi a 16 bit
                        CS_N => '0',           -- Chip Select sempre attivo
                        WE_N => wrt_enable_N,  -- Write enable
                        OE_N => rd_N,          -- Read enable
                        D => data_bus);        -- Bus di dati a 8 bit

  -- Istanzia il Merge_Bits, isTTYData e isTTYStatus in un std_logic_vector(1 downto 0)
  blk_Merge_Bits: Merge_Bits port map(b0 => isTTYStatus,
                                      b1 => isTTYData,
                                      Q  => TTY_select);

  -- Istanzia il MUX a 4 ingressi (8 bit di dati da default del generic)
  blk_MUX4: MUX4 port map(sel => TTY_select,       -- Data input select: bit1 = isTTYData, bit0 = isTTYStatus
                          D0  => data_bus,         -- Input #0 = Data bus interno (bus dati RAM)
                          D1  => TTY_status,       -- Input #1 = Stato del TTY (bit1 = RTS, bit0 = DTR)
                          D2  => TTY_data,         -- Input #2 = Data received da TTY
                          D3  => (others => '0'),  -- Input #3 = non utilizzato (tutto a '0')
                          Q   => Mem_out);         -- Output del MUX4 nell'output dati del blocco Main_Memory

  -- Istanzia un buffer a tre stati (8 bit di dati da default del generic)
  blk_Buf: BufZ port map(OE => rd_N,       -- Output enable, attivo quando rd_N  disattivo
                         D  => Mem_in,     -- Input dati del blocco Main_Memory
                         Q  => data_bus);  -- Output sul data bus interno (bus dati RAM)

end struct;

--------------------------------------------------------
