--------------------------------------------------------
-- VHDL code per modulo REG16.
-- By Sebastiano Otupacca - PAP1 2008/2009, 09-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity REG16 is

  port(CK    : in std_logic;                        -- Clock
       CLR_N : in std_logic;                        -- Clear (active-low)
       WA_N  : in std_logic;                        -- Write A (active-low)
       ENA_N : in std_logic;                        -- Enable A (active-low)
       ENB_N : in std_logic;                        -- Enable B (active-low)
       SA    : in std_logic_vector(3 downto 0);     -- Select A
       SB    : in std_logic_vector(3 downto 0);     -- Select B
       DIN   : in std_logic_vector(15 downto 0);    -- Data input
       A     : out std_logic_vector(15 downto 0);   -- A output
       B     : out std_logic_vector(15 downto 0));  -- B output

end REG16;

--------------------------------------------------------

architecture struct of REG16 is

  component NOT1
    port(D : in std_logic;
         Q : out std_logic);
  end component;

  component NOTx
    generic(n: natural := 8);
    port(D : in std_logic_vector(n-1 downto 0);
         Q : out std_logic_vector(n-1 downto 0));
  end component;

  component Decoder
    generic(n: natural := 2);
    port(EN_N : in std_logic;
         D    : in std_logic_vector(n-1 downto 0);
         Q    : out std_logic_vector(2**n-1 downto 0));
  end component;

  component REG4
    port(CK    : in std_logic;                        -- Clock
         CLR_N : in std_logic;                        -- Clear (active-low)
         WA_N  : in std_logic;                        -- Write A (active-low)
         ENA_N : in std_logic;                        -- Enable A (active-low)
         ENB_N : in std_logic;                        -- Enable B (active-low)
         SA    : in std_logic_vector(1 downto 0);     -- Select A
         SB    : in std_logic_vector(1 downto 0);     -- Select B
         DIN   : in std_logic_vector(15 downto 0);    -- Data input
         A     : out std_logic_vector(15 downto 0);   -- A output
         B     : out std_logic_vector(15 downto 0));  -- B output
  end component;

  component ZREG4
    port(CK    : in std_logic;                        -- Clock
         CLR_N : in std_logic;                        -- Clear (active-low)
         WA_N  : in std_logic;                        -- Write A (active-low)
         ENA_N : in std_logic;                        -- Enable A (active-low)
         ENB_N : in std_logic;                        -- Enable B (active-low)
         SA    : in std_logic_vector(1 downto 0);     -- Select A
         SB    : in std_logic_vector(1 downto 0);     -- Select B
         DIN   : in std_logic_vector(15 downto 0);    -- Data input
         A     : out std_logic_vector(15 downto 0);   -- A output
         B     : out std_logic_vector(15 downto 0));  -- B output
  end component;

  -- Output dei decoder
  signal decoderA   : std_logic_vector(3 downto 0);
  signal decoderA_N : std_logic_vector(3 downto 0);
  signal decoderB   : std_logic_vector(3 downto 0);
  signal decoderB_N : std_logic_vector(3 downto 0);

  signal ENA : std_logic;
  signal ENB : std_logic;

begin

  -- Istanzia la porta NOT di negazione di ENA_N
  blk_NOT_A: NOT1 port map(D => ENA_N,
                           Q => ENA);

  -- Istanzia il decoder A
  blk_DecoderA: Decoder port map(EN_N => ENA,
                                 D    => SA(3 downto 2),
                                 Q    => decoderA);

  -- Istanzia la porta NOT di negazione di ENB_N
  blk_NOT_B: NOT1 port map(D => ENB_N,
                           Q => ENB);

  -- Istanzia il decoder B
  blk_DecoderB: Decoder port map(EN_N => ENB,
                                 D    => SB(3 downto 2),
                                 Q    => decoderB);

  -- Istanzia la porta NOT a 4 bit di negazione del segnale decoderA
  blk_NOTxA: NOTx generic map(n => 4)
                  port map(D => decoderA,
                           Q => decoderA_N);

  -- Istanzia la porta NOT a 4 bit di negazione del segnale decoderB
  blk_NOTxB: NOTx generic map(n => 4)
                  port map(D => decoderB,
                           Q => decoderB_N);

  -- Istanzia il modulo ZREG4 (RF0)
  blk_R0: ZREG4 port map(CK    => CK,
                         CLR_N => CLR_N,
                         WA_N  => WA_N,
                         ENA_N => decoderA_N(0),
                         ENB_N => decoderB_N(0),
                         SA    => SA(1 downto 0),
                         SB    => SB(1 downto 0),
                         DIN   => DIN,
                         A     => A,
                         B     => B);

  -- Istanzia il modulo REG4 (RF1)
  blk_R1: REG4 port map(CK    => CK,
                        CLR_N => CLR_N,
                        WA_N  => WA_N,
                        ENA_N => decoderA_N(1),
                        ENB_N => decoderB_N(1),
                        SA    => SA(1 downto 0),
                        SB    => SB(1 downto 0),
                        DIN   => DIN,
                        A     => A,
                        B     => B);

  -- Istanzia il modulo REG4 (RF2)
  blk_R2: REG4 port map(CK    => CK,
                        CLR_N => CLR_N,
                        WA_N  => WA_N,
                        ENA_N => decoderA_N(2),
                        ENB_N => decoderB_N(2),
                        SA    => SA(1 downto 0),
                        SB    => SB(1 downto 0),
                        DIN   => DIN,
                        A     => A,
                        B     => B);

  -- Istanzia il modulo REG4 (RF3)
  blk_R3: REG4 port map(CK    => CK,
                        CLR_N => CLR_N,
                        WA_N  => WA_N,
                        ENA_N => decoderA_N(3),
                        ENB_N => decoderB_N(3),
                        SA    => SA(1 downto 0),
                        SB    => SB(1 downto 0),
                        DIN   => DIN,
                        A     => A,
                        B     => B);

end struct;

--------------------------------------------------------
