--------------------------------------------------------
-- VHDL code per modulo REG4.
-- By Sebastiano Otupacca - PAP1 2008/2009, 09-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------

entity REG4 is

  port(CK    : in std_logic;                        -- Clock
       CLR_N : in std_logic;                        -- Clear (active-low)
       WA_N  : in std_logic;                        -- Write A (active-low)
       ENA_N : in std_logic;                        -- Enable A (active-low)
       ENB_N : in std_logic;                        -- Enable B (active-low)
       SA    : in std_logic_vector(1 downto 0);     -- Select A
       SB    : in std_logic_vector(1 downto 0);     -- Select B
       DIN   : in std_logic_vector(15 downto 0);    -- Data input
       A     : out std_logic_vector(15 downto 0);   -- A output
       B     : out std_logic_vector(15 downto 0));  -- B output

end REG4;

--------------------------------------------------------

architecture struct of REG4 is

  component NOT1
    port(D : in std_logic;
         Q : out std_logic);
  end component;

  component NOTx
    generic(n: natural := 8);
    port(D : in std_logic_vector(n-1 downto 0);
         Q : out std_logic_vector(n-1 downto 0));
  end component;

  component OR2
    port(A : in std_logic;
         B : in std_logic;
         Q : out std_logic);
  end component;

  component Decoder
    generic(n: natural := 2);
    port(EN_N : in std_logic;
         D    : in std_logic_vector(n-1 downto 0);
         Q    : out std_logic_vector(2**n-1 downto 0));
  end component;

  component Reg
    generic(n: natural := 8);
    port(clk   : in std_logic;
         CLR_N : in std_logic;
         EN_N  : in std_logic;
         D     : in std_logic_vector(n-1 downto 0);
         Q     : out std_logic_vector(n-1 downto 0));
  end component;

  component MUX4
    generic(n: natural := 8);
    port(sel : in std_logic_vector(1 downto 0);
         D0  : in std_logic_vector(n-1 downto 0);
         D1  : in std_logic_vector(n-1 downto 0);
         D2  : in std_logic_vector(n-1 downto 0);
         D3  : in std_logic_vector(n-1 downto 0);
         Q   : out std_logic_vector(n-1 downto 0));
  end component;

  component BufZ
    generic(n: natural := 8);
    port(OE : in std_logic;
         D  : in std_logic_vector(n-1 downto 0);
         Q  : out std_logic_vector(n-1 downto 0));
  end component;

  -- Abilitazione del decoder
  signal DecoderEnable : std_logic;

  -- Output del decoder
  signal decoderOut   : std_logic_vector(3 downto 0);
  signal decoderOut_N : std_logic_vector(3 downto 0);

  -- Outputs dei registri R0, R1, R2 e R3
  signal R0_Out : std_logic_vector(15 downto 0);
  signal R1_Out : std_logic_vector(15 downto 0);
  signal R2_Out : std_logic_vector(15 downto 0);
  signal R3_Out : std_logic_vector(15 downto 0);

  -- Outputs dei MUX_A e MUX_B
  signal MUX_A_Out : std_logic_vector(15 downto 0);
  signal MUX_B_Out : std_logic_vector(15 downto 0);

  signal ENA : std_logic;
  signal ENB : std_logic;

begin

  -- Istanzia la OR a 2 input
  blk_OR2: OR2 port map(A => WA_N,
                        B => ENA_N,
                        Q => DecoderEnable);

  -- Istanzia il decoder
  blk_Decoder: Decoder port map(EN_N => DecoderEnable,
                                D    => SA,
                                Q    => decoderOut);

  -- Istanzia una porta NOT a 4 bit
  blk_NOTx: NOTx generic map(n => 4)
                 port map(D => decoderOut,
                          Q => decoderOut_N);

  -- Istanzia il Registro R0
  blk_R0: Reg generic map(n => 16)
              port map (clk   => CK,
                        CLR_N => CLR_N,
                        EN_N  => decoderOut_N(0),
                        D     => DIN,
                        Q     => R0_Out);

  -- Istanzia il Registro R1
  blk_R1: Reg generic map(n => 16)
              port map (clk   => CK,
                        CLR_N => CLR_N,
                        EN_N  => decoderOut_N(1),
                        D     => DIN,
                        Q     => R1_Out);

  -- Istanzia il Registro R2
  blk_R2: Reg generic map(n => 16)
              port map (clk   => CK,
                        CLR_N => CLR_N,
                        EN_N  => decoderOut_N(2),
                        D     => DIN,
                        Q     => R2_Out);

  -- Istanzia il Registro R3
  blk_R3: Reg generic map(n => 16)
              port map (clk   => CK,
                        CLR_N => CLR_N,
                        EN_N  => decoderOut_N(3),
                        D     => DIN,
                        Q     => R3_Out);

  -- Istanzia il MUX per l'output A
  blk_MUX_A: MUX4 generic map(n => 16)
                  port map(sel => SA,
                           D0  => R0_Out,
                           D1  => R1_Out,
                           D2  => R2_Out,
                           D3  => R3_Out,
                           Q   => MUX_A_Out);

  -- Istanzia il MUX per l'output B
  blk_MUX_B: MUX4 generic map(n => 16)
                  port map(sel => SB,
                           D0  => R0_Out,
                           D1  => R1_Out,
                           D2  => R2_Out,
                           D3  => R3_Out,
                           Q   => MUX_B_Out);

  -- Istanzia la porta NOT di negazione di ENA_N
  blk_NOTxA: NOT1 port map(D => ENA_N,
                           Q => ENA);

  -- Istanzia il buffer 3-state dell'output A
  blk_Buf_A: BufZ generic map(n => 16)
                  port map(OE => ENA,
                           D  => MUX_A_Out,
                           Q  => A);

  -- Istanzia la porta NOT di negazione di ENB_N
  blk_NOTxB: NOT1 port map(D => ENB_N,
                           Q => ENB);

  -- Istanzia il buffer 3-state dell'output B
  blk_Buf_B: BufZ generic map(n => 16)
                  port map(OE => ENB,
                           D  => MUX_B_Out,
                           Q  => B);

end struct;

--------------------------------------------------------
