--------------------------------------------------------
-- VHDL code per blocco TTY Control del modulo MEMORY.
-- By Sebastiano Otupacca - PAP1 2008/2009, 06-05-2009
--------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

--------------------------------------------------------

entity TTY_Control is

  port(ck             : in std_logic;                      -- Clock
       CLR_N          : in std_logic;                      -- Clear (active-low)
       wrt_N          : in std_logic;                      -- Write enable (active-low)
       Data_in        : in std_logic_vector(7 downto 0);   -- Data input (8 bit)
       Addr_in        : in std_logic_vector(15 downto 0);  -- Bus indirizzi (16 bit)
       DTR_in         : in std_logic;                      -- Data Terminal Ready dal TTY
       RTS_in         : in std_logic;                      -- Request To Send dal TTY
       TTY_data_recv  : in std_logic_vector(7 downto 0);   -- Dati ricevuti dal TTY
       TTY_statusAddr : in std_logic_vector(15 downto 0);  -- Indirizzo di mappatura del TTY_Status nella memoria
       TTY_dataAddr   : in std_logic_vector(15 downto 0);  -- Indirizzo di mappatura del TTY_Data nella memoria
       DSR_out        : out std_logic;                     -- Data Set Ready verso il TTY
       CTS_out        : out std_logic;                     -- Clear To Send verso il TTY
       TTY_status_out : out std_logic_vector(7 downto 0);  -- Status del TTY
       TTY_data_send  : out std_logic_vector(7 downto 0);  -- Dati inviati tramite il TTY
       TTY_data_thru  : out std_logic_vector(7 downto 0);  -- Dati ricevuti dal TTY e passati al blocco Main_Memory
       isTTYStatus    : out std_logic;                     -- '1' se l'indirizzo  mappato per la lettura/scrittura dello stato TTY
       isTTYData      : out std_logic);                    -- '1' se l'indirizzo  mappato per la lettura/scrittura del Data TTY

end TTY_Control;

--------------------------------------------------------

architecture struct of TTY_Control is

  component NOT1
    port(D : in std_logic;
         Q : out std_logic);
  end component;

  component MUX2
    generic(n: natural := 8);
    port(sel : in std_logic;
         D0  : in std_logic_vector(n-1 downto 0);
         D1  : in std_logic_vector(n-1 downto 0);
         Q   : out std_logic_vector(n-1 downto 0));
  end component;

  component Reg
    generic(n: natural := 8);
    port(clk   : in std_logic;
         CLR_N : in std_logic;
         EN_N  : in std_logic;
         D     : in std_logic_vector(n-1 downto 0);
         Q     : out std_logic_vector(n-1 downto 0));
  end component;

  component AND2
    port(A : in std_logic;
         B : in std_logic;
         Q : out std_logic);
  end component;

  component XOR2_Serie
    generic(n: natural := 8);
    port(A : in std_logic_vector(n-1 downto 0);
         B : in std_logic_vector(n-1 downto 0);
         Q : out std_logic_vector(n-1 downto 0));
  end component;

  component OR16
    port(D : in std_logic_vector(15 downto 0);
         Q : out std_logic);
  end component;

  component BufferPort
    port(D : in std_logic;
         Q : out std_logic);
  end component;

  component Merge_Vectors
    generic(nV0: natural := 8; nV1: natural := 8);
    port(v0 : in std_logic_vector(nV0-1 downto 0);        -- Vector low
         v1 : in std_logic_vector(nV1-1 downto 0);        -- Vector high
         Q  : out std_logic_vector(nV0+nV1-1 downto 0));  -- Vector merge output
  end component;

  component Merge_Bits
    port(b0 : in std_logic;
         b1 : in std_logic;
         Q  : out std_logic_vector(1 downto 0));
  end component;

  -- Output delle serie di XOR di confronto indirizzi
  signal XOR_status_out : std_logic_vector(15 downto 0);
  signal XOR_data_out   : std_logic_vector(15 downto 0);

  -- Abilitazione registro Data per il TTY
  signal Data_enable   : std_logic;
  signal Data_enable_N : std_logic;

  -- Segnali bufferizzati
  signal isTTYStatusBuf : std_logic;
  signal isTTYDataBuf : std_logic;

  -- Segnali per lo status TTY
  signal Status_select   : std_logic;
  signal Status_select_N : std_logic;
  signal Status_source   : std_logic_vector(1 downto 0);
  signal Status_out      : std_logic_vector(1 downto 0);

  -- Segnale DTR + RTS
  signal DTR_RTS : std_logic_vector(1 downto 0);

  signal wrt : std_logic;

begin

  -- Istanzia la serie di 16 XOR a 2 input per il confronto dell'indirizzo
  -- di mappatura in memoria dello Status del TTY
  blk_Status: XOR2_Serie generic map(n => 16)
                         port map(A => TTY_statusAddr,
                                  B => Addr_in,
                                  Q => XOR_status_out);

  -- Istanzia la OR a 16 input per il test dell'indirizzo di mappatura in
  -- memoria dello status del TTY
  blk_OR_status: OR16 port map(D => XOR_status_out,
                               Q => isTTYStatusBuf);

  -- Istanzia il buffer per il bit di mappatura dello Status TTY
  blk_BufStatus: BufferPort port map(D => isTTYStatusBuf,
                                     Q => isTTYStatus);

  -- Istanzia la serie di 16 XOR a 2 input per il confronto dell'indirizzo
  -- di mappatura in memoria del Data del TTY
  blk_Data: XOR2_Serie generic map(n => 16)
                       port map(A => TTY_dataAddr,
                                B => Addr_in,
                                Q => XOR_data_out);

  -- Istanzia la OR a 16 input per il test dell'indirizzo di mappatura in
  -- memoria del Data del TTY
  blk_OR_data: OR16 port map(D => XOR_data_out,
                             Q => isTTYDataBuf);

  -- Istanzia il buffer per il bit di mappatura del Data TTY
  blk_BufData: BufferPort port map(D => isTTYDataBuf,
                                   Q => isTTYData);

  -- Istanzia la porta NOT di negazione del segnale wrt_N
  blk_NOT_W: NOT1 port map(D => wrt_N,
                           Q => wrt);

  -- Istanzia la porta AND a 2 input per la logica di abilitazione alla
  -- scrittura dello status del TTY
  blk_ANDstatus: AND2 port map(A => isTTYStatusBuf,
                               B => wrt,
                               Q => Status_select);

  -- Istanzia la porta AND a 2 input per la logica di abilitazione alla
  -- scrittura dei dati del TTY
  blk_ANDdata: AND2 port map(A => isTTYDataBuf,
                             B => wrt,
                             Q => Data_enable);

  -- Istanzia la porta NOT di negazione del segnale Status_select
  blk_NOT_S: NOT1 port map(D => Status_select,
                           Q => Status_select_N);

  -- Istanzia il MUX di data input (solo i primi 2 bit)
  blk_MUXstatus: MUX2 generic map(n => 2)
                      port map(sel => Status_select_N,      -- Selector tra Data_in e 0
                               D0  => Data_in(1 downto 0),  -- Input #0 = Data input, bit1 = CTS, bit0 = DSR
                               D1  => (others => '0'),      -- Input #1 = tutti i bit a 0
                               Q   => Status_source);       -- Output

  -- Istanzia il registro di Status TTY
  blk_StatusReg: Reg generic map(n => 2)
                     port map(clk   => ck,             -- Clock
                              CLR_N => CLR_N,          -- Clear (globale)
                              EN_N  => '0',            -- Registro sempre abilitato
                              D     => Status_source,  -- Dall'uscita del MUX blk_MUXstatus (2 bit)
                              Q     => Status_out);    -- Status, bit1 = CTS, bit0 = DSR

  -- Istanzia il buffer per il bit Data Set Ready
  blk_DSR: BufferPort port map(D => Status_out(0),
                               Q => DSR_out);

  -- Istanzia il buffer per il bit Clear To Send
  blk_CTS: BufferPort port map(D => Status_out(1),
                               Q => CTS_out);

  -- Istanzia la porta NOT di negazione del segnale Data_enable
  blk_NOT_D: NOT1 port map(D => Data_enable,
                           Q => Data_enable_N);

  -- Istanzia il registro di Data TTY
  blk_DataReg: Reg port map(clk   => ck,              -- Clock
                            CLR_N => CLR_N,           -- Clear (globale)
                            EN_N  => Data_enable_N,   -- Abilitazione del registro
                            D     => Data_in,         -- Data input
                            Q     => TTY_data_send);  -- All'input dati del blocco Main_Memory

  -- Istanzia il Merge Bits, DTR concatenato RTS in un vector di 2 bit
  blk_DTR_RTS: Merge_Bits port map(b0 => DTR_in,
                                   b1 => RTS_in,
                                   Q  => DTR_RTS);

  -- Istanzia il Merge Vectors, DTR e RTS in un vector di 8 bit
  blk_StatusByte: Merge_Vectors generic map(nV0 => 2, nV1 => 6)
                                port map(v0 => DTR_RTS,
                                         v1 => (others => '0'),
                                         Q  => TTY_status_out);

  -- Forward dei dati TTY in ricezione (verso il blocco Main_Memory)
  TTY_data_thru <= TTY_data_recv;

end struct;

--------------------------------------------------------
